function [sys,x0,str,ts] = baloo_AEKF(t,x,u,flag,J,Res,Laf,L,B,R,Q,Lambda,K,ThetaMAX,Beta,m1,m2,I_init,W_init,Ta_init,thetao)
% baloo_AEKF(t,x,u,flag,J,Res,Laf,L,B,R,Q,Lambda,K,ThetaMAX,Beta,m,I_init,W_init,Ta_init,thetao)
% This is main program of the adaptive-gain extended kalman filter
% See: E.BUSVELLE, N.BOIZOT  for details.
%
%  S-FUNCTION PARAMETERS : J,Res,Laf1,L,B : model parameters
%                          R, Q : weigth matrices of the Riccati equation
%                          Lambda, K, ThetaMAX, Beta, m1, m2 : parameters of the observer
%                          theta0, I_init, W_init, Tl_init : initial value
%                                           before any change of variables
%                     
%  S-FUNCTION INPUTS : u(1) = system output (I)
%                      u(2) = V (system input)
%                      u(3) = Innovation     
% 
%  S-FUNCTION OUTPUTS : y(1) = I estimated
%                       y(2) = wr estimated
%                       y(3) = Tl estimated
%                       y(4) = theta
%
%       -----------------------
% IMPORTANT : the change of coordinates imply that this observer works only for strictly positive
% values of 'I' !!
%       -----------------------
% NOTE : this S-function uses the SymReshape.m file !!
%       -----------------------

% last modified by Elnycko, 24/07/2007

switch flag,
  case 0
    [sys,x0,str,ts]=mdlInitializeSizes(thetao,I_init,W_init,Ta_init);
  case 1
    sys = mdlDerivatives(t,x,u,J,Res,Laf,L,B,R,Q,Lambda,K,ThetaMAX,Beta,m1,m2); 
  case 3
    sys = mdlOutputs(t,x,u);   
  case { 2, 4, 9 } 
    sys = [];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]); 
end
% End of csfunc.


function [sys,x0,str,ts] = mdlInitializeSizes(thetao,I_init,W_init,Ta_init)

sizes = simsizes;
sizes.NumContStates  = 3+4*3/2+1; % state + ricatti equation + theta
sizes.NumDiscStates  = 0;
sizes.NumOutputs     =  3+1; % 3+1; % states + theta + ricatti
sizes.NumInputs      = 3;   % Ia + V + Innovation
sizes.DirFeedthrough = 0;     
sizes.NumSampleTimes = 1;
sys = simsizes(sizes);
% zo=[0.2;0.2;0];
% zo = [  3.2974
%     5.6994e+003
%      0];
zo=[I_init
    I_init*W_init
    I_init*Ta_init];


% zo=[0.1;50;0];

Po=SymReshape(eye(3,3));
% Po= [ 0.7089   -388.4562   -0.7071    2.1343e+005    401.1706   1.3513]';
%  Po=1.0e+004 *[ 0.0001   -0.0176   -0.0011    2.2001    0.1357    0.0121]';
x0= [zo;Po;thetao];

str = [];

ts = [0 0];
% End of mdlInitializeSizes.


function sys = mdlDerivatives(t,x,u,J,Res,Laf,L,B,R,Q,Lambda,K,ThetaMAX,Beta,m1,m2)
% Inputs
y=u(1);
V=u(2);
In=u(3);

% States
z1=x(1);
z2=x(2);
z3=x(3);
z=[z1;z2;z3];

% Parameters
m=m1+m2;

P=SymReshape(x(4:9));
theta = x(10);

Rtheta = (1/theta)*R;
Idelta = diag([1  theta  theta^2]);
Qtheta =theta*Idelta*Q*Idelta;
A= [0     -Laf/L      0 
    0        0       -1/J
    0        0         0];

b=[-(Res/L)*z1+V/L;
   -(B/J)*z2+(Laf/J)*z1^3-(Res/L)*z2-(Laf/L)*(z2^2/z1)+(V/L)*(z2/z1)
    (V/L)*(z3/z1)-(Res/L)*z3-(Laf/L)*(z2*z3/z1)];

B11 = -Res/L;
B21 = 3*(Laf/J)*z1^2 + (Laf/L)*(z2^2/z1^2) - (V/L)*(z2/z1^2);
B22 = -(B/J) -(Res/L) - 2*(Laf/L)*(z2/z1) + (V/L)*(1/z1);
B31 = -(V/L)*(z3/z1^2) + (Laf/L)*(z2*z3/z1^2);
B32 = -(Laf/L)*(z3/z1);
B33 = (V/L)*(1/z1) -(Res/L) -(Laf/L)*(z2/z1);

bstar = [     B11             0       0
              B21            B22      0
              B31            B32      B33 ];
C=[1 0 0];

            
sys(1:3) = A*z+b-P*C'*inv(Rtheta)*(C*z-y);
sys(4:9) = SymReshape(P*(A+bstar)'+(A+bstar)*P - P*C'*inv(Rtheta)*C*P +Qtheta);

sig=1./(1+exp(-Beta*(In-m)));
sys(10) = Lambda*(1-sig)*(1-theta)+K*(ThetaMAX-theta)*sig;
% End of mdlDerivatives.


function sys = mdlOutputs(t,x,u)
sys =[x(1)
      x(2)/x(1)
      x(3)/x(1)
      x(10)]; 
  % End of mdlOutputs.