function [sys,x0,str,ts] = balooDC(t,x,u,flag,Wo,Io,J,Res,Laf,L,B)
% balooDC(t,x,u,flag,Wo,Io,J,Res,Laf,L,B)
% is a  2 input / 2 ouput S-function that simulates a 
% Series connected DC motor modelised by the academic equations :
%
% d(I)/dt = (u - Res*I -Laf1 I.wr )/L     (current) 
% d(x2)/dt = (Laf2 I^2 -B.wr - Tl) / J  (motor acceleration)
% Where u is the unput voltage and Tl the torque load
% -B.wr is the frictions torque
%
% S-FUNCTION PARAMETERS : Wo   : motor speed nominal value (rad.s^-1)
%                         Io   : current nominal value (Amps)
%                         J    : system inertia
%                         Res  : total circuit resistance = armature + field (Ohms)
%                         Laf  : Inductance coefficient of the rotor (Henry)
%                         L    : Inductance coefficient of the series branch (Henry) + Inductance coefficient of the armarture branch (H) 
%                         B    : Friction torque coefficient  
%
% S-FUNCTION INPUTS :     u(1) = voltage
%                         u(2) = Tl, torque load (perturbations)
%                        
%
% S-FUNCTION OUTPUTS: armature current (I) & rotation speed (wr)

% last modified by Elnycko, 24/07/2007
switch flag,
  case 0
    [sys,x0,str,ts]=mdlInitializeSizes(Wo,Io);
  case 1
    sys = mdlDerivatives(t,x,u,J,Res,Laf,L,B); 
  case 3
    sys = mdlOutputs(t,x,u); 
  
  case { 2, 4, 9 } 
    sys = [];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]); 
end
% End of csfunc.

function [sys,x0,str,ts] = mdlInitializeSizes(Wo,Io)
sizes = simsizes;
sizes.NumContStates  = 2;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 2;
sizes.NumInputs      = 1+1; %(V, Ta)
sizes.DirFeedthrough = 0;     
sizes.NumSampleTimes = 1;
sys = simsizes(sizes);

x0 = [Io;Wo];  
str = [];

ts = [0 0];
% End of mdlInitializeSizes.

function sys = mdlDerivatives(t,x,u,J,Res,Laf,L,B)
V =u(1);
Ta=u(2);
I=x(1);    
wr=x(2);

sys = [(1/L)*(V-Res*I-Laf*wr*I)
        (1/J)*(-B*wr+Laf*I^2-Ta)]; 
%end,
    
function sys = mdlOutputs(t,x,u)
sys = x;
% End of mdlOutputs.